#include "StdAfx.h"
#include "AESCrypto.h"

#include "openssl/aes.h"
#include "openssl/evp.h"

typedef struct _pwdcrypt_t
{
	EVP_CIPHER_CTX ctx;
	char key[16];
	char iv[16];
} pwdcrypt_t;

CAESCrypto::CAESCrypto(void)
{
}

CAESCrypto::~CAESCrypto(void)
{
}


// key_base_table  seed  ̿ؼ crypt_key  .
void * CAESCrypto::pwdcrypt_init(const char * seed, const char * key_base_table, int key_base_table_len)
{
	int i_key;
	int i_seed;
	int seed_len = strlen(seed);
	pwdcrypt_t * phandle = (pwdcrypt_t *)malloc(sizeof(pwdcrypt_t));

	if (NULL == phandle)
	{
		return NULL;
	}

	memset(phandle, 0, sizeof(pwdcrypt_t));
	EVP_CIPHER_CTX_init(&phandle->ctx);

	for(i_key = 0, i_seed = 0 ;
		i_key < (int)sizeof(phandle->key) ;
		i_key++, (i_seed = (i_seed + 1) % seed_len))
	{
		int index_of_table = seed[i_seed] % key_base_table_len;
		phandle->key[i_key] = key_base_table[ index_of_table ];
	}

	return (void *)phandle;
}

void CAESCrypto::pwdcrypt_free(void * handle)
{
	pwdcrypt_t * p = (pwdcrypt_t *)handle;
	EVP_CIPHER_CTX_cleanup(&p->ctx);

	free((pwdcrypt_t *)handle);
}

bool CAESCrypto::pwdcrypt_encrypt(const char * data, int data_len,
					  unsigned char * pOutBuffer, int * pOutBufferLen,
					  const char * seed,
					  const char * key_base_table, int key_base_table_len)
{
	if(data_len == 0)
	{
		*pOutBufferLen = 0;
		return true;
	}

	pwdcrypt_t * phandle = (pwdcrypt_t *)pwdcrypt_init(seed, key_base_table, key_base_table_len);

	if (NULL == phandle)
	{
		return false;
	}

	EVP_EncryptInit(&phandle->ctx, EVP_aes_128_cbc(), (const unsigned char *)phandle->key, (unsigned char *)phandle->iv);

	EVP_EncryptUpdate(&phandle->ctx, pOutBuffer, pOutBufferLen, (const unsigned char *)data, data_len);

	int remainLen = 0;
	EVP_EncryptFinal(&phandle->ctx, pOutBuffer + *pOutBufferLen, &remainLen);

	(*pOutBufferLen) += remainLen;

	pwdcrypt_free(phandle);
	return true;
}

bool CAESCrypto::pwdcrypt_decrypt(const unsigned char * data, int data_len,
					  char * pOutBuffer, int * pOutBufferLen,
					  const char * seed,
					  const char * key_base_table, int key_base_table_len)
{
	if(data_len == 0)
	{
		*pOutBufferLen = 0;
		return true;
	}
    
	pwdcrypt_t * phandle = (pwdcrypt_t *)pwdcrypt_init(seed, key_base_table, key_base_table_len);

	if (NULL == phandle)
	{
		return false;
	}
	
	EVP_DecryptInit(&phandle->ctx, EVP_aes_128_cbc(), (const unsigned char *)phandle->key, (unsigned char *)phandle->iv);

	EVP_DecryptUpdate(&phandle->ctx, (unsigned char *)pOutBuffer, pOutBufferLen, data, data_len);
	
	int remainLen = 0;
	EVP_DecryptFinal(&phandle->ctx, (unsigned char *)pOutBuffer + *pOutBufferLen, &remainLen);

	*pOutBufferLen += remainLen;
	pOutBuffer[*pOutBufferLen] = '\0';

	pwdcrypt_free(phandle);
	return true;
}
